"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bindings_rewrite_1 = require("./bindings-rewrite");
var crypto_1 = require("crypto");
var fs_1 = require("fs");
var path_1 = require("path");
function hashName(name) {
    return crypto_1.createHash('md5').update(name).digest('hex').toString().slice(0, 8);
}
function default_1(options) {
    return new NativeModulePlugin(options);
}
exports.default = default_1;
var NativeModulePlugin = (function () {
    function NativeModulePlugin(options) {
        this.options = options;
        this.limit2Project = false;
        this.modules = Object.keys(options);
        this.test = new RegExp("node_modules/(" + Object.keys(options).join('|') + ").*.js|.node$");
    }
    NativeModulePlugin.prototype.init = function (context) {
        context.allowExtension('.node');
    };
    NativeModulePlugin.prototype.transform = function (file) {
        file.loadContents();
        if (file.absPath.endsWith('.node')) {
            var contents = fs_1.readFileSync(file.absPath);
            var module_1 = this.modules.find(function (x) {
                return Boolean(~file.absPath.indexOf(path_1.join('node_modules', x)));
            });
            var bindingName = path_1.basename(file.absPath);
            var settings = this.options[module_1];
            var moduleDir = hashName(contents);
            file.contents = "\n      var fs=require('fs');var path=require('path');var binding='" + contents.toString('base64') + "';function mkdirp(r,t){t=t||null,r=path.resolve(r);try{fs.mkdirSync(r),t=t||r}catch(c){if(\"ENOENT\"===c.code)t=mkdirp(path.dirname(r),t),mkdirp(r,t);else{var i;try{i=fs.statSync(r)}catch(r){throw c}if(!i.isDirectory())throw c}}return t};";
            if (settings === true) {
                file.contents += ("\n          mkdirp('" + moduleDir + "');\n          var bindingPath = path.join(process.cwd(), '" + moduleDir + "', '" + bindingName + "')\n          require('fs').writeFileSync(bindingPath, Buffer.from(binding, 'base64'))\n          process.dlopen(module, bindingPath)\n        ").trim();
                return;
            }
            var depth_1 = 0;
            settings.additionalFiles.forEach(function (file) {
                var ownDepth = 0;
                file.split('/').forEach(function (x) { return x === '..' && ownDepth++; });
                depth_1 = ownDepth > depth_1 ? ownDepth : depth_1;
            });
            var segments = [moduleDir];
            while (depth_1--) {
                segments.push(hashName(moduleDir + depth_1));
            }
            segments.push(bindingName);
            file.contents += "\n        var cwd = process.cwd()\n        var bindingFileParts = " + JSON.stringify(segments) + ";\n        var bindingFile = path.join.apply(path, [cwd].concat(bindingFileParts));\n        mkdirp(path.dirname(bindingFile));\n        fs.writeFileSync(bindingFile, Buffer.from(binding, 'base64'));\n        " + settings.additionalFiles.reduce(function (code, filename, i) {
                var contents = fs_1.readFileSync(path_1.join(path_1.dirname(file.absPath), filename));
                return (code += "\n            var file" + i + " = '" + contents.toString('base64') + "';\n            var filePath" + i + " = path.join(cwd, bindingFileParts[0], '" + filename
                    .split('../')
                    .join('') + "');\n            mkdirp(path.dirname(filePath" + i + "));\n            fs.writeFileSync(filePath" + i + ", Buffer.from(file" + i + ", 'base64'));\n          ");
            }, '') + ";\n        process.dlopen(module, bindingFile)\n      ";
            return;
        }
        var bindingsRewrite = new bindings_rewrite_1.BindingsRewrite();
        file.makeAnalysis(null, {
            plugins: [
                {
                    onNode: function (file, node, parent) {
                        bindingsRewrite.onNode(file.absPath, node, parent);
                    },
                    onEnd: function (file) {
                        if (bindingsRewrite.rewrite) {
                            var index = file.analysis.dependencies.indexOf('bindings');
                            if (~index) {
                                file.analysis.dependencies.splice(index, 1);
                            }
                            (_a = file.analysis.dependencies).push.apply(_a, bindingsRewrite.nativeModulePaths);
                            file.analysis.requiresRegeneration = true;
                        }
                        var _a;
                    }
                }
            ]
        });
    };
    return NativeModulePlugin;
}());
exports.NativeModulePlugin = NativeModulePlugin;
