"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var Bluebird = require("bluebird");
var fs_1 = require("fs");
var util_1 = require("./util");
function readStreamAsync(stream) {
    return new Bluebird(function (resolve) {
        var input = '';
        stream.setEncoding('utf-8');
        stream.on('data', function (x) {
            input += x;
        });
        stream.once('end', function () { return resolve(util_1.dequote(input)); });
        stream.resume && stream.resume();
    });
}
/**
 * The "cli" step detects whether the process is in a tty. If it is then the input is read into memory.
 * Otherwise, it is buffered from stdin. If no input options are passed in the tty, the package.json#main file is used.
 * After all the build steps have run, the output (the executable) is written to a file or piped to stdout.
 *
 * Configuration:
 *   - compiler.options.input - file path to the input bundle.
 *     - fallbacks: stdin, package.json#main
 *   - compiler.options.output - file path to the output executable.
 *     - fallbacks: stdout, nexe_ + epoch + ext
 *
 * @param {*} compiler
 * @param {*} next
 */
function cli(compiler, next) {
    return __awaiter(this, void 0, void 0, function () {
        var _a, input, output, log, _b, _c, bundle, _d, deliverable;
        return __generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    _a = compiler.options, input = _a.input, output = _a.output;
                    log = compiler.log;
                    if (!(!input && !process.stdin.isTTY)) return [3 /*break*/, 2];
                    log.step('Using stdin as input');
                    _b = compiler;
                    return [4 /*yield*/, readStreamAsync(process.stdin)];
                case 1:
                    _b.input = _e.sent();
                    return [3 /*break*/, 7];
                case 2:
                    if (!input) return [3 /*break*/, 4];
                    log.step("Using input file as the main module: " + input);
                    _c = compiler;
                    return [4 /*yield*/, util_1.readFileAsync(path_1.normalize(input), 'utf-8')];
                case 3:
                    _c.input = _e.sent();
                    return [3 /*break*/, 7];
                case 4:
                    if (!!compiler.options.empty) return [3 /*break*/, 6];
                    bundle = require.resolve(process.cwd());
                    log.step("Using the cwd's main file as the main module");
                    _d = compiler;
                    return [4 /*yield*/, util_1.readFileAsync(bundle, 'utf-8')];
                case 5:
                    _d.input = _e.sent();
                    return [3 /*break*/, 7];
                case 6:
                    log.step('Using empty input as the main module');
                    compiler.input = '';
                    _e.label = 7;
                case 7:
                    compiler.input = compiler.input.trim();
                    return [4 /*yield*/, next()];
                case 8:
                    _e.sent();
                    compiler.output = output || "" + compiler.options.name + (util_1.isWindows ? '.exe' : '');
                    return [4 /*yield*/, compiler.compileAsync()];
                case 9:
                    deliverable = _e.sent();
                    return [2 /*return*/, new Bluebird(function (resolve, reject) {
                            var step = log.step('Writing result to file');
                            deliverable
                                .pipe(fs_1.createWriteStream(path_1.normalize(compiler.output)))
                                .on('error', reject)
                                .once('close', function (e) {
                                if (e) {
                                    reject(e);
                                }
                                else if (compiler.output) {
                                    fs_1.chmodSync(compiler.output, '755');
                                    step.log("Executable written to: " + compiler.output);
                                    resolve(compiler.quit());
                                }
                            });
                        })];
            }
        });
    });
}
exports.default = cli;
