"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var parseArgv = require("minimist");
var path_1 = require("path");
var os_1 = require("os");
function padRight(str, l) {
    return (str + ' '.repeat(l)).substr(0, l);
}
var defaults = {
    temp: process.env.NEXE_TEMP || path_1.join(process.cwd(), '.nexe'),
    version: process.version.slice(1),
    flags: [],
    configure: [],
    make: [],
    targets: [],
    vcBuild: ['nosign', 'release', process.arch],
    enableNodeCli: false,
    bundle: true,
    build: true,
    patches: []
};
var alias = {
    i: 'input',
    o: 'output',
    t: 'target',
    n: 'name',
    v: 'version',
    r: 'resource',
    a: 'resource',
    p: 'python',
    f: 'flag',
    c: 'configure',
    m: 'make',
    vc: 'vcBuild',
    s: 'snapshot',
    cli: 'enableNodeCli',
    h: 'help',
    l: 'loglevel'
};
var argv = parseArgv(process.argv, { alias: alias, default: defaults });
exports.argv = argv;
var help = ("\nnexe --help              CLI OPTIONS\n\n  -b   --build                              -- build from source\n  -i   --input      =index.js               -- application entry point\n  -o   --output     =my-app.exe             -- path to output file\n  -t   --target     =win32-x64-6.10.3       -- *target a prebuilt binary\n  -n   --name       =my-app                 -- main app module name\n  -v   --version    =" + padRight(process.version.slice(1), 23) + "-- node version\n  -p   --python     =/path/to/python2       -- python executable\n  -f   --flag       =\"--expose-gc\"          -- *v8 flags to include during compilation\n  -c   --configure  =\"--with-dtrace\"        -- *pass arguments to the configure command\n  -m   --make       =\"--loglevel\"           -- *pass arguments to the make command\n  -vc  --vcBuild    =x64                    -- *pass arguments to vcbuild.bat\n  -s   --snapshot   =/path/to/snapshot      -- build with warmup snapshot\n  -r   --resource   =./paths/**/*           -- *embed file bytes within the binary\n       --bundle     =./path/to/config       -- pass a module path that exports nexeBundle\n       --temp       =./path/to/temp         -- nexe temp files (for downloads and source builds)\n       --no-bundle                          -- set when input is already bundled\n       --ico                                -- file name for alternate icon file (windows)\n       --rc-*                               -- populate rc file options (windows)\n       --clean                              -- force download of sources\n       --enableNodeCli                      -- enable node cli enforcement (blocks app cli)\n       --sourceUrl                          -- pass an alternate source (node.tar.gz) url\n       --silent                             -- disable logging\n       --verbose                            -- set logging to verbose\n\n       -* variable key name                 * option can be used more than once").trim() + os_1.EOL;
function flattenFilter() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return (_a = []).concat.apply(_a, args).filter(function (x) { return x; });
    var _a;
}
/**
 * Extract keys such as { "rc-CompanyName": "Node.js" } to
 * { CompanyName: "Node.js" }
 * @param {*} match
 * @param {*} options
 */
function extractCliMap(match, options) {
    return Object.keys(options)
        .filter(function (x) { return match.test(x); })
        .reduce(function (map, option) {
        var key = option.split('-')[1];
        map[key] = options[option];
        delete options[option];
        return map;
    }, {});
}
function tryResolveMainFileName() {
    var filename;
    try {
        var file = require.resolve(process.cwd());
        filename = path_1.basename(file).replace(path_1.extname(file), '');
    }
    catch (_) { }
    return !filename || filename === 'index' ? 'nexe_' + Date.now() : filename;
}
function extractLogLevel(options) {
    if (options.loglevel)
        return options.loglevel;
    if (options.silent)
        return 'silent';
    if (options.verbose)
        return 'verbose';
    return 'info';
}
function extractName(options) {
    var name = options.name;
    if (typeof options.input === 'string' && !name) {
        name = path_1.basename(options.input).replace(path_1.extname(options.input), '');
    }
    name = name || tryResolveMainFileName();
    return name.replace(/\.exe$/, '');
}
function normalizeOptionsAsync(input) {
    if (argv.help || argv._.some(function (x) { return x === 'version'; })) {
        process.stderr.write(argv.help ? help : '2.0.0-beta.7' + os_1.EOL, function () { return process.exit(0); });
    }
    var options = Object.assign({}, defaults, input);
    var opts = options;
    delete opts._;
    options.loglevel = extractLogLevel(options);
    options.name = extractName(options);
    options.flags = flattenFilter(opts.flag, options.flags);
    options.targets = flattenFilter(opts.target, options.targets);
    options.make = flattenFilter(options.make);
    options.vcBuild = flattenFilter(options.vcBuild);
    options.configure = flattenFilter(options.configure);
    options.resources = flattenFilter(opts.resource, options.resources);
    options.rc = options.rc || extractCliMap(/^rc-.*/, options);
    if (options.build) {
        options.targets = [];
        options.build = true;
    }
    else if (!options.targets.length) {
        var defaultTarget = [process.platform, process.arch, options.version].join('-');
        options.targets = [defaultTarget];
    }
    Object.keys(alias).filter(function (k) { return k !== 'rc'; }).forEach(function (x) { return delete opts[x]; });
    return Promise.resolve(options);
}
exports.normalizeOptionsAsync = normalizeOptionsAsync;
